@@ -3,9 +3,9 @@ use warnings;
 
 package Template::Pure;
 
-our $VERSION = '0.013';
+our $VERSION = '0.014';
 
-use DOM::Tiny;
+use Mojo::DOM58;
 use Scalar::Util;
 use Template::Pure::ParseUtils;
 use Template::Pure::Filters;
@@ -13,6 +13,7 @@ use Template::Pure::DataContext;
 use Template::Pure::DataProxy;
 use Template::Pure::EncodedString;
 use Template::Pure::Iterator;
+use Storable qw(dclone);
 
 sub new {
   my ($proto, %args) = @_;
@@ -20,19 +21,88 @@ sub new {
 
   die '"template" is required' unless $args{template};
 
+  my ($dom, @directives)  = $class->_prepare_dom($args{template});  
+  unshift @directives, @{$args{directives}};
+
   return bless +{
     filters => $args{filters} || +{},
-    template => $args{template},
-    directives => $args{directives},
+    directives => \@directives,
+    dom => $dom, 
   }, $class;
 }
 
+sub _prepare_dom {
+  my ($class, $template) = @_;
+  my @directives = ();
+  my $dom = Mojo::DOM58->new($template);
+
+  my $nodes = $dom->child_nodes;
+  my $placeholder_cnt = 0;
+  my $do; $do = sub {
+    my ($item, $num) = @_;
+    if($item->type eq 'pi') {
+      my ($target, %attrs) = $class->parse_processing_instruction($item->tree->[1]);
+      my $ctx = delete $attrs{ctx};
+      my $src = delete $attrs{src};
+
+      if($target eq 'pure-include') {
+        $item->replace("<span id='include-$placeholder_cnt'>include placeholder</span>");
+        my @include_directives;
+        if($ctx) {
+          @include_directives = ("#include-$placeholder_cnt" => +{ $ctx => ['^.' => "/$src"]});
+        } elsif(%attrs) {
+          @include_directives = ("#include-$placeholder_cnt" => [\%attrs, '^.' => "/$src"]);
+        }else {
+          @include_directives = ("^#include-$placeholder_cnt", $src)
+        }
+        push @directives, @include_directives;
+      } elsif($target eq 'pure-wrapper') {
+        $item->following('*')->first->attr('data-pure-wrapper-id'=>"wrapper-$placeholder_cnt");
+        $item->remove;
+        if($ctx) {
+          push @directives, (
+            "^*[data-pure-wrapper-id=wrapper-$placeholder_cnt]", +{ $ctx => ['^.' => "/$src"]},
+            "*[data-pure-wrapper-id=wrapper-$placeholder_cnt]\@data-pure-wrapper-id", sub { undef },
+          );
+        } elsif(%attrs) {
+          push @directives, (
+            "^*[data-pure-wrapper-id=wrapper-$placeholder_cnt]", [\%attrs, '^.' => "/$src"],
+            "*[data-pure-wrapper-id=wrapper-$placeholder_cnt]\@data-pure-wrapper-id", sub { undef },
+          );
+        }else {
+          push @directives, (
+            "^*[data-pure-wrapper-id=wrapper-$placeholder_cnt]", $src,
+            "*[data-pure-wrapper-id=wrapper-$placeholder_cnt]\@data-pure-wrapper-id", sub { undef },
+          );
+        }
+      } elsif($target eq 'pure-overlay') {
+        $item->following('*')->first->attr('data-pure-overlay-id'=>"overlay-$placeholder_cnt");
+        $item->remove;
+        push @directives, (
+          "^*[data-pure-overlay-id=overlay-$placeholder_cnt]", [ +{%attrs, src=>$src }, '^.' => 'src'],
+          "*[data-pure-overlay-id=overlay-$placeholder_cnt]\@data-pure-overlay-id", sub { undef },
+        );
+      } else {
+        warn "Encountering processing instruction $target that I can't process";
+      }
+      $placeholder_cnt++;
+    }
+    $item->child_nodes->each($do);
+  };
+  $nodes->each($do);
+
+  return ($dom, @directives);
+}
+
+sub clone_dom { return dclone(shift->{dom}) }
+
 sub render {
   my ($self, $data_proto, $extra_directives) = @_;
-  my $dom = DOM::Tiny->new($self->{template});
-  
   $data_proto = Template::Pure::DataProxy->new($data_proto, self=>$self);
+  $extra_directives = [] unless $extra_directives;
 
+  my $dom = $self->clone_dom;
+  
   return $self->process_dom($dom, $data_proto, $extra_directives)->to_string;
 }
 
@@ -50,6 +120,7 @@ sub default_filters { Template::Pure::Filters->all }
 sub parse_match_spec { Template::Pure::ParseUtils::parse_match_spec($_[1]) }
 sub parse_data_spec { Template::Pure::ParseUtils::parse_data_spec($_[1]) }
 sub parse_data_template { Template::Pure::ParseUtils::parse_data_template($_[1]) }
+sub parse_processing_instruction { Template::Pure::ParseUtils::parse_processing_instruction($_[1]) }
 sub parse_itr_spec { Template::Pure::ParseUtils::parse_itr_spec($_[1]) } 
 sub escape_html { Template::Pure::Filters::escape_html($_[1]) }
 sub encoded_string { Template::Pure::EncodedString->new($_[1]) }
@@ -77,7 +148,7 @@ sub _process_dom_recursive {
 
   $self->{root_data} = $data_proto unless exists $self->{root_data};
 
-  my $data =  Template::Pure::DataContext->new($data_proto, $self->{root_data});
+  my $data = Template::Pure::DataContext->new($data_proto, $self->{root_data});
 
   ($data, @directives) = $self->_process_directive_instructions($dom, $data, @directives);
 
@@ -153,7 +224,7 @@ sub _process_obj {
         $self->_process_mode($_, $value, %match_spec);
       });
     }
-  } elsif($obj->isa('DOM::Tiny')) {
+  } elsif($obj->isa('Mojo::DOM58')) {
       $self->_process_match_spec($dom, $obj, %match_spec);
   } else {
     die "Can't process object of type $obj.";
@@ -310,7 +381,7 @@ sub _process_sub_data {
 
 sub _process_iterator {
   my ($self, $dom, $key, $iterator, @actions) = @_; 
-  my $new_dom = DOM::Tiny->new($dom);
+  my $new_dom = Mojo::DOM58->new($dom);
   while(my $datum = $iterator->next) {
     my $new_data = +{
       $key => $datum, 
@@ -347,7 +418,7 @@ sub _value_from_dom {
   if($match_spec{target} eq 'content') {
     #return $self->encoded_string($self->at_or_die($dom, $match_spec{css})->content);
     # Not sure if there is a more effecient way to make this happen...
-    return DOM::Tiny->new($self->at_or_die($dom, $match_spec{css})->content);
+    return Mojo::DOM58->new($self->at_or_die($dom, $match_spec{css})->content);
   } elsif($match_spec{target} eq 'node') {
     ## When we want a full node, with HTML tags, we encode the string
     ## since I presume they want a copy not escaped.  T 'think' this is
@@ -417,7 +488,16 @@ sub _process_mode {
   my $safe_value = $self->escape_html($value);
 
   ## This behavior may be tweaked in the future.
-  return $dom->remove if $self->_value_is_undef($safe_value);
+  if($self->_value_is_undef($safe_value)) {
+    if($target eq 'node') {
+      return $dom->remove;
+    } elsif($target eq 'content') {
+      return $dom->remove; # TODO, or should this remove just the content..?
+    } elsif(my $attr = $$target) {
+      return delete $dom->attr->{$attr};
+
+    }
+  }
 
   if($mode eq 'replace') {
     if($target eq 'content') {
@@ -473,7 +553,7 @@ sub filter {
 
 =head1 NAME
 
-Template::Pure - Perlish Port of pure.js
+Template::Pure - Perlish Port of pure.js and more
 
 =head1 SYNOPSIS
 
@@ -610,7 +690,7 @@ L<Template::Pure> has two required parameters:
 
 =item template
 
-This is a string that is an HTML template that can be parsed by L<DOM::Tiny>
+This is a string that is an HTML template that can be parsed by L<Mojo::DOM58>
 
 =item directives
 
@@ -642,8 +722,8 @@ always return to it, as you will later see in the L</DIRECTIVES> section.
 
 =item process_dom ($data, ?\@extra_directives?)
 
-Works just like 'render', except we return a L<DOM::Tiny> object instead of a string directly.
-Useful if you wish to retrieve the L<DOM::Tiny> object for advanced, custom tranformations.
+Works just like 'render', except we return a L<Mojo::DOM58> object instead of a string directly.
+Useful if you wish to retrieve the L<Mojo::DOM58> object for advanced, custom tranformations.
 
 =item data_at_path ($data, $path)
 
@@ -876,7 +956,7 @@ provide the matched value.  This anonymous subroutine receives the following thr
 arguments:
 
     $instance: The template instance
-    $dom: The DOM Node at the current match (as a L<DOM::Tiny> object).
+    $dom: The DOM Node at the current match (as a L<Mojo::DOM58> object).
     $data: Data reference at the current context.
 
 Your just need to return the value desired which will substitute for the matched node's
@@ -1430,7 +1510,7 @@ Results in:
 Lastly you can mimic a type of inheritance using data mapping and
 node aliasing:
 
-   my $master_html = q[
+   my $overlay_html = q[
       <html>
         <head>
           <title>Example Title</title>
@@ -1447,8 +1527,8 @@ node aliasing:
       </html>
     ];
 
-    my $master = Template::Pure->new(
-      template=>$master_html,
+    my $overlay = Template::Pure->new(
+      template=>$overlay_html,
       directives=> [
         'title' => 'title',
         '^title+' => 'scripts',
@@ -1484,7 +1564,7 @@ node aliasing:
             scripts => \'^head script',
             content => \'body',
           },
-          '^.' => $master,
+          '^.' => $overlay,
         ]
       ]);
 
@@ -1520,10 +1600,10 @@ Results in:
       </body>
     </html>
 
-=head2 Object - A DOM::Tiny instance
+=head2 Object - A Mojo::DOM58 instance
 
 In the case where you set the value of the action target to an instance of
-L<DOM::Tiny>, we let the value of that perform the replacement indicated by
+L<Mojo::DOM58>, we let the value of that perform the replacement indicated by
 the match specification:
 
     my $html = q[
@@ -1540,7 +1620,7 @@ the match specification:
     my $pure = Template::Pure->new(
       template=>$html,
       directives=> [
-        'p' => DOM::Tiny->new("<a href='localhost:foo'>Foo!</a>"),
+        'p' => Mojo::DOM58->new("<a href='localhost:foo'>Foo!</a>"),
       ]);
 
     my $data = +{
@@ -1620,6 +1700,8 @@ Results in:
       </body>
     </html>
 
+B<NOTE> For an alternative method see L</PROCESSING INSTRUCTIONS>
+
 =head2 Using Dot Notation in Directive Data Mapping
 
 L<Template::Pure> allows you to indicate a path to a point in your
@@ -1896,7 +1978,7 @@ not just the value.  Can be combined with '+' for append/prepend.
 
 =item '|': Run a filter on the current node
 
-Passed the currently selected node to a code reference.  You can run L<DOM::Tiny>
+Passed the currently selected node to a code reference.  You can run L<Mojo::DOM58>
 transforms on the entire selected node.  Nothing should be returned from this 
 coderef.
 
@@ -1962,9 +2044,392 @@ to non programmers.
 
 See L<Template::Pure::Filters> for all bundled filters.
 
+=head1 PROCESSING INSTRUCTIONS
+
+Generally L<Template::Pure> proposes its best to keep your actual HTML templates as simple
+and valid as possible, instead putting your transformations and data binding logic into
+directives.  This leads to a strong separate of responsibilities and prevents your templates
+from getting messy.  However there are a few situations where we'd like to offer the template
+designer some options to control the overall template structure and to encapsulate common
+design elements or template rules.  For example its common in a website to have some common
+layouts that set overall page structure and import common CSS and Javascript libraries.  Additionally
+its common to have 'snippets' of HTML that are shared across lots of documents (such as common
+header or footer elements, or advertizements panels, etc.)  You can describe these via directives
+but in order to empower designers and reduce your directive complexity L<Template::Pure> allowes
+one to insert HTML Processing instructions into your templates that get parsed when the template
+object is instantiated and added as additional directives.  This allows one to create directives
+declaratively in the template, rather than programtically in your code.
+
+The availability of this feature in no way suggests that one approach or the other is best.  You
+should determine that based on your team and project needs.
+
+L<Template::Pure> currently offers the following three processing instructions, and does not
+yet offer an API to create your own.  This may change in the future.
+
+B<NOTE> All processing instructions are parsed and evaluated during instantiation of your
+template object and all generated directives are adding to the end of your existing ones.  As
+a result these instructions are run last.
+
+=head1 Includes
+
+Allows one to inject a template render into a placeholder spot in the current template.  Example:
+
+    my $include_html = qq[
+      <span id="footer">Copyright&nbsp;</span>];
+
+    my $include = Template::Pure->new(
+      template=>$include_html,
+      directives=> [
+        '#footer+' => 'copyright_year',
+      ]);
+
+    my $base_html = q[
+      <html>
+        <head>
+          <title>Page Title: </title>
+        </head>
+        <body>
+          <div id='story'>Example Story</div>
+          <?pure-include src='foot_include' copyright_year='meta.copyright.year'?>
+        </body>
+      </html>
+    ];
+
+    my $base = Template::Pure->new(
+      template => $base_html,
+      directives => [
+        'title+' => 'meta.title',
+        '#story' => 'story',
+      ]
+    );
+
+    print $base->render({
+      story => 'It was a dark and stormy night...',
+      foot_include => $include
+      meta => {
+        title=>'Dark and Stormy..',
+        copyright => {
+          year => 2016,
+          author=>'jnap'}
+        }
+      },
+    });
+
+Returns:
+
+      <html>
+        <head>
+          <title>Page Title: Dark and Stormy..'</title>
+        </head>
+        <body>
+          <div id='story'>It was a dark and stormy night...</div>
+          <span id="footer">Copyright 2016</span>
+        </body>
+      </html>
+
+This is basically the same as:
+
+    my $base_html = q[
+      <html>
+        <head>
+          <title>Page Title: </title>
+        </head>
+        <body>
+          <div id='story'>Example Story</div>
+          <span id='footer'>...</span>
+        </body>
+      </html>
+    ];
+
+    my $base = Template::Pure->new(
+      template => $base_html,
+      directives => [
+        'title+' => 'meta.title',
+        '#story' => 'story',
+        '^#footer' => 'foot_include',
+      ]
+    );
+
+    print $base->render({
+      story => 'It was a dark and stormy night...',
+      foot_include => $include
+      meta => {
+        title=>'Dark and Stormy..',
+        copyright => {
+          year => 2016,
+          author=>'jnap'}
+        }
+      },
+    });
+
+Or alternatively (if you don't want to allow one to alter the include via
+processing data):
+
+    my $base_html = q[
+      <html>
+        <head>
+          <title>Page Title: </title>
+        </head>
+        <body>
+          <div id='story'>Example Story</div>
+          <span id='footer'>...</span>
+        </body>
+      </html>
+    ];
+
+    my $base = Template::Pure->new(
+      template => $base_html,
+      directives => [
+        'title+' => 'meta.title',
+        '#story' => 'story',
+        '^#footer' => $include,
+      ]
+    );
+
+    print $base->render({
+      story => 'It was a dark and stormy night...',
+      meta => {
+        title=>'Dark and Stormy..',
+        copyright => {
+          year => 2016,
+          author=>'jnap'}
+        }
+      },
+    });
+
+Basically you set the processing directive and the PI is fully replaced by the referenced
+template.  Format is like:
+
+  <?pure-include src=$data_path @args?>
+
+Where 'src' must be a data context path (see L<\Using Dot Notation in Directive Data Mapping>
+for more on referencing a data path) that is an instance of L<Template::Pure> and @args are
+a list of mappings to import data into the target include from the calling instance current
+data context.  Alternatively, you may set a data context root instead using 'ctx' as an
+argument:
+
+    my $include_html = qq[
+      <span id="footer">Copyright&nbsp;</span>];
+
+    my $include = Template::Pure->new(
+      template=>$include_html,
+      directives=> [
+        '#footer+' => 'copyright.year',
+      ]);
+
+    ...
+    <?pure-include src='foot_include' ctx='meta'?>
+    ...
+
+This might be the preferred method when you wish to copy a full section of data to your
+target include.  You may not combine the 'ctx' method and the named args method.
+
+If you do not specify a 'ctx' or named args, we default to a context of the root data
+context.  This probably leaks too much information into your include but is not terrible
+for prototyping.
+
+=head2 Wrapper
+
+Similar to the include processing instruction, it provides template authors with a declaritive
+approach to L</Object - Set the match value to another Pure Template>.  Example:
+
+    my $story_section_wrapper_html = qq[
+      <section>
+        <h1>story title</h1>
+        <p>By:&nbsp;</p>
+      </section>];
+
+    my $story_section_wrapper = Template::Pure->new(
+      template=>$story_section_wrapper_html,
+      directives=> [
+        'h1' => 'title',
+        'p+' => 'author',
+        '^p+' => 'content',
+      ]);
+
+    my $base_html = q[
+      <html>
+        <head>
+          <title>Page Title:&nbsp;</title>
+        </head>
+        <body>
+          <?pure-wrapper src='section_wrapper' ctx='meta'?>
+          <div id='story'>Example Story</div>
+        </body>
+      </html>
+    ];
+
+    my $base = Template::Pure->new(
+      template=>$base_html,
+      directives=> [
+        'title+' => 'meta.title',
+        '#story' => 'story,
+      ]
+    );
+
+    print $base->render({
+      story => 'Once Upon a Time...',
+      section_wrapper => $story_section_wrapper,
+      meta => {
+        title=>'Once',
+        author=>'jnap',
+      },
+    });
+
+Results in:
+
+    <html>
+      <head>
+        <title>Page Title:&nbsp;Once</title>
+      </head>
+      <body>
+        <section>
+          <h1>Once</h1>
+          <p>By:&nbsp;jnap</p>
+          <div id='story'>Once Upon a Time</div>
+        </section>
+      </body>
+    </html>
+
+This processing instructions 'wraps' the following tag node with the template that
+is the target of 'src'.  Like L</Includes> you may pass data via named parameters or
+by setting a new data context, as in the given example.
+
+Similar approach using directives only:
+
+    my $base = Template::Pure->new(
+      template=>$base_html,
+      directives=> [
+        'title+' => 'meta.title',
+        '#story' => 'story,
+        '^#story => $story_section_wrapper,
+      ]
+    );
+
+=head2 Overlay
+
+An overlay replaces the selected node with the results on another template.  Typically
+you will pass selected nodes of the original template as directives to the new template.
+This can be used to minic features like template inheritance, that exist in other templating
+systems.  One example:
+
+    my $overlay_html = q[
+      <html>
+        <head>
+          <title>Example Title</title>
+          <link rel="stylesheet" href="/css/pure-min.css"/>
+            <link rel="stylesheet" href="/css/grids-responsive-min.css"/>
+              <link rel="stylesheet" href="/css/common.css"/>
+          <script src="/js/3rd-party/angular.min.js"></script>
+            <script src="/js/3rd-party/angular.resource.min.js"></script>
+        </head>
+        <body>
+        </body>
+      </html>
+    ];
+
+    my $overlay = Template::Pure->new(
+      template=>$overlay_html,
+      directives=> [
+        'title' => 'title',
+        'head+' => 'scripts',
+        'body' => 'content',
+      ]);
+
+    my $base_html = q[
+      <?pure-overlay src='layout'
+        title=\'title'
+        scripts=\'^head script' 
+        content=\'body'?>
+      <html>
+        <head>
+          <title>Page Title:&nbsp;</title>
+          <script>
+          function foo(bar) {
+            return baz;
+          }
+          </script>
+        </head>
+        <body>
+          <div id='story'>Example Story</div>
+        </body>
+      </html>
+    ];
+
+    my $base = Template::Pure->new(
+      template=>$base_html,
+      directives=> [
+        'title+' => 'meta.title',
+        '#story' => 'story,
+      ]
+    );
+
+    print $base->render({
+      layout => $overlay,
+      story => 'Once Upon a Time...',
+      meta => {
+        title=>'Once',
+        author=>'jnap',
+      },
+    });
+
+Renders As:
+
+    <html>
+      <head>
+        <title>Once</title>
+        <link rel="stylesheet" href="/css/pure-min.css"/>
+          <link rel="stylesheet" href="/css/grids-responsive-min.css"/>
+            <link rel="stylesheet" href="/css/common.css"/>
+        <script src="/js/3rd-party/angular.min.js"></script>
+          <script src="/js/3rd-party/angular.resource.min.js"></script>
+            <script>
+            function foo(bar) {
+              return baz;
+            }
+            </script>
+      </head>
+      <body>
+        <div id='story'>Once Upon a Time...</div>
+      </body>
+    </html>
+
+The syntax of the processing instruction is:
+
+    <?pure-overlay src='' @args ?>
+
+Where 'src' is a data path to the template you want to use as the overlay, and @args is
+a list of key values which populate the data context of the overlay when you process it.
+Often these values will be references to existing nodes in the base template (as in the
+examples \'title' and \'body' above) but they can also be used to map values from your
+data context in the same way we do so for L</Include> and L</Wrapper>.
+
+If you were to write this as 'directives only' it would look like:
+
+    my $base = Template::Pure->new(
+      template=>$base_html,
+      directives=> [
+        'title+' => 'meta.title',
+        '#story' => 'story,
+        'html' => [
+          {
+            title => \'title'
+            script s=> \'^head script' 
+            content => \'body'
+          },
+          '^.' => 'layout',
+        ],
+      ]
+    );
+
+Please note that although in this example the overlay wrapped over the entire template, it is
+not limited to that, rather like the L</Wrapper> processing instruction it just takes the next
+tag node following as its overlay target.  So you could have more than one overlap in a document
+and can overlay sections for those cases where a L</Wrapper> is not sufficently complex.
+
 =head1 IMPORTANT NOTE REGARDING VALID HTML
 
-Please note that L<DOM::Tiny> tends to enforce rule regarding valid HTML5.  For example, you
+Please note that L<Mojo::DOM58> tends to enforce rule regarding valid HTML5.  For example, you
 cannot nest a block level element inside a 'P' element.  This might at time lead to some
 surprising results in your output.
 
@@ -1974,7 +2439,7 @@ John Napiorkowski L<email:jjnapiork@cpan.org>
   
 =head1 SEE ALSO
  
-L<DOM::Tiny>, L<HTML::Zoom>.  Both of these are approaches to programmatically examining and
+L<Mojo::DOM58, L<HTML::Zoom>.  Both of these are approaches to programmatically examining and
 altering a DOM.
 
 L<Template::Semantic> is a similar system that uses XPATH instead of a CSS inspired matching
